@@ -73,14 +73,15 @@
 ! LICENSE: GNU GPL (version 2 or later)
 ! 
 ! AUTHORS:
-!    Yi Yang (yiyang@umn.edu) and Hui Zou (hzou@stat.umn.edu), 
-!    School of Statistics, University of Minnesota.
+!    * Yi Yang (yi.yang6@mcgill.ca) and + Hui Zou (hzou@stat.umn.edu), 
+!    * Department of Mathematics and Statistics, McGill University
+!    + School of Statistics, University of Minnesota.
 ! 
 ! REFERENCES:
-!    Yang, Y. and Zou, H. (2012). 
+!    Yang, Y. and Zou, H. (2015). 
 !    A Fast Unified Algorithm for Computing Group-Lasso Penalized Learning Problems
-!    Journal of Computational and Graphical Statistics.
-!    Under review.
+!    Statistics and Computing.
+!    25(6), 1129-1141.
 
 ! --------------------------------------------------
 SUBROUTINE ls_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
@@ -127,16 +128,15 @@ SUBROUTINE ls_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
     DOUBLE PRECISION::al
     DOUBLE PRECISION::alf
     DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: b
-    DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE ::oldbeta
-    DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE ::r
-    DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE ::oldb
-    DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE ::u
-    DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE ::dd
+    DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: oldbeta
+    DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: r
+    DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: oldb
+    DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: u
+    DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE :: dd
     INTEGER, DIMENSION (:), ALLOCATABLE :: oidx
     INTEGER:: g
     INTEGER::j
     INTEGER::l
-    INTEGER::ierr
     INTEGER::ni
     INTEGER::me
     INTEGER::start
@@ -150,14 +150,10 @@ SUBROUTINE ls_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
     DOUBLE PRECISION:: vl(nvars)
     DOUBLE PRECISION:: al0
 ! - - - allocate variables - - -
-    ALLOCATE(b(0:nvars),STAT=jerr)
-    ALLOCATE(oldbeta(0:nvars),STAT=ierr)
-    jerr=jerr+ierr
-    ALLOCATE(r(1:nobs),STAT=ierr)
-    jerr=jerr+ierr
-    ALLOCATE(oidx(1:bn),STAT=ierr)
-    jerr=jerr+ierr
-    IF(jerr/=0) RETURN
+    ALLOCATE(b(0:nvars))
+    ALLOCATE(oldbeta(0:nvars))
+    ALLOCATE(r(1:nobs))
+    ALLOCATE(oidx(1:bn))
 ! - - - checking pf - - -
     IF(maxval(pf) <= 0.0D0) THEN
         jerr=10000
@@ -169,12 +165,13 @@ SUBROUTINE ls_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
     al = 0.0D0
     mnl = Min (mnlam, nlam)
     r = y
-    b=0.0D0
-    oldbeta=0.0D0
-    idx=0
-    oidx=0
-    npass=0
-    ni=npass
+    b = 0.0D0
+    oldbeta = 0.0D0
+    idx = 0
+    oidx = 0
+    npass = 0
+    ni = npass
+    alf = 0.0D0
 ! --------- lambda loop ----------------------------
     IF(flmin < 1.0D0) THEN
         flmin = Max (mfl, flmin)
@@ -182,8 +179,7 @@ SUBROUTINE ls_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
     ENDIF
     vl = matmul(r, x)/nobs
     DO g = 1,bn
-            ALLOCATE(u(bs(g)),STAT=ierr)
-            IF(ierr/=0) RETURN
+            ALLOCATE(u(bs(g)))
             u = vl(ix(g):iy(g))
             ga(g) = sqrt(dot_product(u,u))
             DEALLOCATE(u)
@@ -229,13 +225,9 @@ SUBROUTINE ls_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
                     IF(jxx(g) == 0) CYCLE
                     start=ix(g)
                     end=iy(g)
-                    ALLOCATE(u(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    ALLOCATE(dd(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    ALLOCATE(oldb(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    IF(jerr/=0) RETURN
+                    ALLOCATE(u(bs(g)))
+                    ALLOCATE(dd(bs(g)))
+                    ALLOCATE(oldb(bs(g)))
                     oldb=b(start:end)
                     u=matmul(r,x(:,start:end))/nobs
                     u=gam(g)*b(start:end)+u
@@ -259,14 +251,14 @@ SUBROUTINE ls_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
                     ENDIF
                     DEALLOCATE(u,dd,oldb)
                 ENDDO
-				IF(intr /= 0) THEN
-	                d=sum(r)/nobs
-	                IF(d/=0.0D0) THEN
-	                    b(0)=b(0)+d
-	                    r=r-d
-	                    dif=max(dif,d**2)
-	                ENDIF
-				ENDIF
+                IF(intr /= 0) THEN
+                    d=sum(r)/nobs
+                    IF(d/=0.0D0) THEN
+                        b(0)=b(0)+d
+                        r=r-d
+                        dif=max(dif,d**2)
+                    ENDIF
+                ENDIF
                 IF (ni > pmax) EXIT
                 IF (dif < eps) EXIT
                 IF(npass > maxit) THEN
@@ -281,13 +273,9 @@ SUBROUTINE ls_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
                         g=idx(j)
                         start=ix(g)
                         end=iy(g)
-                        ALLOCATE(u(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        ALLOCATE(dd(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        ALLOCATE(oldb(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        IF(jerr/=0) RETURN
+                        ALLOCATE(u(bs(g)))
+                        ALLOCATE(dd(bs(g)))
+                        ALLOCATE(oldb(bs(g)))
                         oldb=b(start:end)
                         u=matmul(r,x(:,start:end))/nobs
                         u=gam(g)*b(start:end)+u
@@ -305,14 +293,14 @@ SUBROUTINE ls_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
                         ENDIF
                         DEALLOCATE(u,dd,oldb)
                     ENDDO
-					IF(intr /= 0) THEN
-	                    d=sum(r)/nobs
-	                    IF(d/=0.0D0) THEN
-	                        b(0)=b(0)+d
-	                        r=r-d
-	                        dif=max(dif,d**2)
-	                    ENDIF
-				    ENDIF
+                    IF(intr /= 0) THEN
+                        d=sum(r)/nobs
+                        IF(d/=0.0D0) THEN
+                            b(0)=b(0)+d
+                            r=r-d
+                            dif=max(dif,d**2)
+                        ENDIF
+                    ENDIF
                     IF(dif<eps) EXIT
                     IF(npass > maxit) THEN
                         jerr=-l
@@ -329,8 +317,7 @@ SUBROUTINE ls_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
             vl = matmul(r, x)/nobs
             DO g = 1, bn
                 IF(jxx(g) == 1) CYCLE
-                ALLOCATE(u(bs(g)),STAT=ierr)
-                IF(ierr/=0) RETURN
+                ALLOCATE(u(bs(g)))
                 u = vl(ix(g):iy(g))
                 ga(g) = sqrt(dot_product(u,u))
                 IF(ga(g) > al*pf(g))THEN
@@ -425,7 +412,6 @@ SUBROUTINE log_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
     INTEGER:: g
     INTEGER::j
     INTEGER::l
-    INTEGER::ierr
     INTEGER::ni
     INTEGER::me
     INTEGER::start
@@ -439,14 +425,10 @@ SUBROUTINE log_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
     DOUBLE PRECISION:: al0
 ! - - - begin - - -
 ! - - - allocate variables - - -
-    ALLOCATE(b(0:nvars),STAT=jerr)
-    ALLOCATE(oldbeta(0:nvars),STAT=ierr)
-    jerr=jerr+ierr
-    ALLOCATE(r(1:nobs),STAT=ierr)
-    jerr=jerr+ierr
-    ALLOCATE(oidx(1:bn),STAT=ierr)
-    jerr=jerr+ierr
-    IF(jerr/=0) RETURN
+    ALLOCATE(b(0:nvars))
+    ALLOCATE(oldbeta(0:nvars))
+    ALLOCATE(r(1:nobs))
+    ALLOCATE(oidx(1:bn))
 ! - - - checking pf - - -
     IF(maxval(pf) <= 0.0D0) THEN
         jerr=10000
@@ -464,6 +446,7 @@ SUBROUTINE log_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
     oidx=0
     npass=0
     ni=npass
+    alf = 0.0D0
 ! --------- lambda loop ----------------------------
     IF(flmin < 1.0D0) THEN
         flmin = Max (mfl, flmin)
@@ -471,8 +454,7 @@ SUBROUTINE log_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
     ENDIF
     vl = matmul(y/(1.0D0+exp(r)), x) / nobs
     DO g = 1,bn
-            ALLOCATE(u(bs(g)),STAT=ierr)
-            IF(ierr/=0) RETURN
+            ALLOCATE(u(bs(g)))
             u = vl(ix(g):iy(g))
             ga(g) = sqrt(dot_product(u,u))
             DEALLOCATE(u)
@@ -518,13 +500,9 @@ SUBROUTINE log_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
                     IF(jxx(g) == 0) CYCLE
                     start=ix(g)
                     end=iy(g)
-                    ALLOCATE(u(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    ALLOCATE(dd(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    ALLOCATE(oldb(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    IF(jerr/=0) RETURN
+                    ALLOCATE(u(bs(g)))
+                    ALLOCATE(dd(bs(g)))
+                    ALLOCATE(oldb(bs(g)))
                     oldb=b(start:end)
                     u=matmul(y/(1.0D0+exp(r)),x(:,start:end))/nobs
                     u=gam(g)*b(start:end)+u
@@ -548,15 +526,15 @@ SUBROUTINE log_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
                     ENDIF
                     DEALLOCATE(u,dd,oldb)
                 ENDDO
-				IF(intr /= 0) THEN
-	                d = sum(y/(1.0D0+exp(r)))
-	                d = 4.0D0*d/nobs
-	                IF(d /= 0.0D0) THEN
-	                    b(0)=b(0)+d
-	                    r=r+y*d
-	                    dif=max(dif,d**2)
-	                ENDIF
-				ENDIF
+                IF(intr /= 0) THEN
+                    d = sum(y/(1.0D0+exp(r)))
+                    d = 4.0D0*d/nobs
+                    IF(d /= 0.0D0) THEN
+                        b(0)=b(0)+d
+                        r=r+y*d
+                        dif=max(dif,d**2)
+                    ENDIF
+                ENDIF
                 IF (ni > pmax) EXIT
                 IF (dif < eps) EXIT
                 IF(npass > maxit) THEN
@@ -571,13 +549,9 @@ SUBROUTINE log_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
                         g=idx(j)
                         start=ix(g)
                         end=iy(g)
-                        ALLOCATE(u(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        ALLOCATE(dd(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        ALLOCATE(oldb(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        IF(jerr/=0) RETURN
+                        ALLOCATE(u(bs(g)))
+                        ALLOCATE(dd(bs(g)))
+                        ALLOCATE(oldb(bs(g)))
                         oldb=b(start:end)
                         u=matmul(y/(1.0D0+exp(r)),x(:,start:end))/nobs
                         u=gam(g)*b(start:end)+u
@@ -595,15 +569,15 @@ SUBROUTINE log_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
                         ENDIF
                         DEALLOCATE(u,dd,oldb)
                     ENDDO
-					IF(intr /= 0) THEN
-	                    d = sum(y/(1.0D0+exp(r)))
-	                    d = 4.0D0*d/nobs
-	                    IF(d/=0.0D0) THEN
-	                        b(0)=b(0)+d
-	                        r=r+y*d
-	                        dif=max(dif, d**2)
-	                    ENDIF
-					ENDIF
+                    IF(intr /= 0) THEN
+                        d = sum(y/(1.0D0+exp(r)))
+                        d = 4.0D0*d/nobs
+                        IF(d/=0.0D0) THEN
+                            b(0)=b(0)+d
+                            r=r+y*d
+                            dif=max(dif, d**2)
+                        ENDIF
+                    ENDIF
                     IF(dif<eps) EXIT
                     IF(npass > maxit) THEN
                         jerr=-l
@@ -620,8 +594,7 @@ SUBROUTINE log_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam,&
             vl = matmul(y/(1.0D0+exp(r)), x) / nobs
             DO g = 1, bn
                 IF(jxx(g) == 1) CYCLE
-                ALLOCATE(u(bs(g)),STAT=ierr)
-                IF(ierr/=0) RETURN
+                ALLOCATE(u(bs(g)))
                 u = vl(ix(g):iy(g))
                 ga(g) = sqrt(dot_product(u,u))
                 IF(ga(g) > al*pf(g))THEN
@@ -713,11 +686,10 @@ SUBROUTINE hsvm_f (delta,bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin
     DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE ::u
     DOUBLE PRECISION, DIMENSION (:), ALLOCATABLE ::dd
     INTEGER, DIMENSION (:), ALLOCATABLE :: oidx
-    INTEGER:: i
+    INTEGER::i
     INTEGER::g
     INTEGER::j
     INTEGER::l
-    INTEGER::ierr
     INTEGER::ni
     INTEGER::me
     INTEGER::start
@@ -731,14 +703,10 @@ SUBROUTINE hsvm_f (delta,bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin
     DOUBLE PRECISION:: al0
 ! - - - begin - - -
 ! - - - allocate variables - - -
-    ALLOCATE(b(0:nvars), STAT = jerr)
-    ALLOCATE(oldbeta(0:nvars), STAT = ierr)
-    jerr=jerr + ierr
-    ALLOCATE(r(1:nobs), STAT = ierr)
-    jerr=jerr + ierr
-    ALLOCATE(oidx(1:bn), STAT = ierr)
-    jerr=jerr + ierr
-    IF(jerr /= 0) RETURN
+    ALLOCATE(b(0:nvars))
+    ALLOCATE(oldbeta(0:nvars))
+    ALLOCATE(r(1:nobs))
+    ALLOCATE(oidx(1:bn))
 ! - - - checking pf - - -
     IF(maxval(pf) <= 0.0D0) THEN
         jerr=10000
@@ -756,6 +724,7 @@ SUBROUTINE hsvm_f (delta,bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin
     oidx = 0
     npass = 0
     ni = npass
+    alf = 0.0D0
 ! --------- lambda loop ----------------------------
     IF(flmin < 1.0D0) THEN
         flmin = Max (mfl, flmin)
@@ -764,8 +733,7 @@ SUBROUTINE hsvm_f (delta,bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin
     vl = 0.0
     CALL hsvmdrv(delta,nobs,nvars,x,y,r,vl)
     DO g = 1,bn
-            ALLOCATE(u(bs(g)),STAT=ierr)
-            IF(ierr/=0) RETURN
+            ALLOCATE(u(bs(g)))
             u = vl(ix(g):iy(g))
             ga(g) = sqrt(dot_product(u,u))
             DEALLOCATE(u)
@@ -811,13 +779,9 @@ SUBROUTINE hsvm_f (delta,bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin
                     IF(jxx(g) == 0) CYCLE
                     start=ix(g)
                     end=iy(g)
-                    ALLOCATE(u(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    ALLOCATE(dd(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    ALLOCATE(oldb(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    IF(jerr/=0) RETURN
+                    ALLOCATE(u(bs(g)))
+                    ALLOCATE(dd(bs(g)))
+                    ALLOCATE(oldb(bs(g)))
                     oldb=b(start:end)
                     u = 0.0D0
                     DO i = 1,nobs
@@ -851,25 +815,25 @@ SUBROUTINE hsvm_f (delta,bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin
                     ENDIF
                     DEALLOCATE(u,dd,oldb)
                 ENDDO
-				IF(intr /= 0) THEN
-	                d = 0.0D0
-	                DO i = 1,nobs
-	                    IF (r(i) > 1.0D0) THEN
-	                        dl(i) = 0.0D0
-	                    ELSEIF (r(i) <= (1-delta)) THEN
-	                        dl(i) = 1.0D0
-	                    ELSE
-	                        dl(i) = (1.0D0 - r(i)) / delta
-	                    ENDIF
-	                    d = d + dl(i)*y(i)
-	                ENDDO
-	                d = 0.5 * delta * d / nobs
-	                IF(d/=0.0D0) THEN
-	                    b(0)=b(0)+d
-	                    r=r+y*d
-	                    dif=max(dif, d**2)
-	                ENDIF
-				ENDIF
+                IF(intr /= 0) THEN
+                    d = 0.0D0
+                    DO i = 1,nobs
+                        IF (r(i) > 1.0D0) THEN
+                            dl(i) = 0.0D0
+                        ELSEIF (r(i) <= (1-delta)) THEN
+                            dl(i) = 1.0D0
+                        ELSE
+                            dl(i) = (1.0D0 - r(i)) / delta
+                        ENDIF
+                        d = d + dl(i)*y(i)
+                    ENDDO
+                    d = 0.5 * delta * d / nobs
+                    IF(d/=0.0D0) THEN
+                        b(0)=b(0)+d
+                        r=r+y*d
+                        dif=max(dif, d**2)
+                    ENDIF
+                ENDIF
                 IF (ni > pmax) EXIT
                 IF (dif < eps) EXIT
                 IF(npass > maxit) THEN
@@ -884,13 +848,9 @@ SUBROUTINE hsvm_f (delta,bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin
                         g=idx(j)
                         start=ix(g)
                         end=iy(g)
-                        ALLOCATE(u(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        ALLOCATE(dd(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        ALLOCATE(oldb(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        IF(jerr/=0) RETURN
+                        ALLOCATE(u(bs(g)))
+                        ALLOCATE(dd(bs(g)))
+                        ALLOCATE(oldb(bs(g)))
                         oldb=b(start:end)
                         u = 0.0D0
                         DO i = 1,nobs
@@ -918,25 +878,25 @@ SUBROUTINE hsvm_f (delta,bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin
                         ENDIF
                         DEALLOCATE(u,dd,oldb)
                     ENDDO
-					IF(intr /= 0) THEN
-	                    d = 0.0D0
-	                    DO i = 1,nobs
-	                        IF (r(i) > 1.0D0) THEN
-	                            dl(i) = 0.0D0
-	                        ELSEIF (r(i) <= (1-delta)) THEN
-	                            dl(i) = 1.0D0
-	                        ELSE
-	                            dl(i) = (1.0D0 - r(i)) / delta
-	                        ENDIF
-	                        d = d + dl(i)*y(i)
-	                    ENDDO
-	                    d = 0.5 * delta * d / nobs
-	                    IF(d/=0.0D0) THEN
-	                        b(0)=b(0)+d
-	                        r=r+y*d
-	                        dif=max(dif, d**2)
-	                    ENDIF
-					ENDIF
+                    IF(intr /= 0) THEN
+                        d = 0.0D0
+                        DO i = 1,nobs
+                            IF (r(i) > 1.0D0) THEN
+                                dl(i) = 0.0D0
+                            ELSEIF (r(i) <= (1-delta)) THEN
+                                dl(i) = 1.0D0
+                            ELSE
+                                dl(i) = (1.0D0 - r(i)) / delta
+                            ENDIF
+                            d = d + dl(i)*y(i)
+                        ENDDO
+                        d = 0.5 * delta * d / nobs
+                        IF(d/=0.0D0) THEN
+                            b(0)=b(0)+d
+                            r=r+y*d
+                            dif=max(dif, d**2)
+                        ENDIF
+                    ENDIF
                     IF(dif<eps) EXIT
                     IF(npass > maxit) THEN
                      jerr=-l
@@ -953,8 +913,7 @@ SUBROUTINE hsvm_f (delta,bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin
             CALL hsvmdrv(delta,nobs,nvars,x,y,r,vl)
             DO g = 1, bn
                 IF(jxx(g) == 1) CYCLE
-                ALLOCATE(u(bs(g)),STAT=ierr)
-                IF(ierr/=0) RETURN
+                ALLOCATE(u(bs(g)))
                 u = vl(ix(g):iy(g))
                 ga(g) = sqrt(dot_product(u,u))
                 IF(ga(g) > al*pf(g))THEN
@@ -1076,7 +1035,6 @@ SUBROUTINE sqsvm_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam
     INTEGER:: g
     INTEGER::j
     INTEGER::l
-    INTEGER::ierr
     INTEGER::ni
     INTEGER::me
     INTEGER::start
@@ -1090,14 +1048,10 @@ SUBROUTINE sqsvm_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam
     DOUBLE PRECISION:: al0
 ! - - - begin - - -
 ! - - - allocate variables - - -
-    ALLOCATE(b(0:nvars),STAT=jerr)
-    ALLOCATE(oldbeta(0:nvars),STAT=ierr)
-    jerr=jerr+ierr
-    ALLOCATE(r(1:nobs),STAT=ierr)
-    jerr=jerr+ierr
-    ALLOCATE(oidx(1:bn),STAT=ierr)
-    jerr=jerr+ierr
-    IF(jerr/=0) RETURN
+    ALLOCATE(b(0:nvars))
+    ALLOCATE(oldbeta(0:nvars))
+    ALLOCATE(r(1:nobs))
+    ALLOCATE(oidx(1:bn))
 ! - - - checking pf - - -
     IF(maxval(pf) <= 0.0D0) THEN
         jerr=10000
@@ -1115,6 +1069,7 @@ SUBROUTINE sqsvm_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam
     oidx=0
     npass=0
     ni=npass
+    alf = 0.0D0
 ! --------- lambda loop ----------------------------
     IF(flmin < 1.0D0) THEN
         flmin = Max (mfl, flmin)
@@ -1123,8 +1078,7 @@ SUBROUTINE sqsvm_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam
     dl = 2.0D0 * dim (1.0D0, r)
     vl = matmul(dl*y, x) / nobs
     DO g = 1,bn
-            ALLOCATE(u(bs(g)),STAT=ierr)
-            IF(ierr/=0) RETURN
+            ALLOCATE(u(bs(g)))
             u = vl(ix(g):iy(g))
             ga(g) = sqrt(dot_product(u,u))
             DEALLOCATE(u)
@@ -1170,15 +1124,11 @@ SUBROUTINE sqsvm_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam
                     IF(jxx(g) == 0) CYCLE
                     start=ix(g)
                     end=iy(g)
-                    ALLOCATE(u(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    ALLOCATE(dd(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    ALLOCATE(oldb(bs(g)),STAT=ierr)
-                    jerr=jerr+ierr
-                    IF(jerr/=0) RETURN
+                    ALLOCATE(u(bs(g)))
+                    ALLOCATE(dd(bs(g)))
+                    ALLOCATE(oldb(bs(g)))
                     oldb=b(start:end)
-                    dl = 2.0 * dim(1.0, r)
+                    dl = 2.0D0 * dim(1.0D0, r)
                     u=matmul(y*dl,x(:,start:end))/nobs
                     u=gam(g)*b(start:end)+u
                     unorm=sqrt(dot_product(u,u))
@@ -1201,16 +1151,16 @@ SUBROUTINE sqsvm_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam
                     ENDIF
                     DEALLOCATE(u,dd,oldb)
                 ENDDO
-				IF(intr /= 0) THEN
-	                dl = 2.0 * dim(1.0, r)
-	                d = dot_product(y,dl)
-	                d = 0.25*d/nobs
-	                IF(d /= 0.0D0) THEN
-	                    b(0)=b(0)+d
-	                    r=r+y*d
-	                    dif=max(dif,d**2)
-	                ENDIF
-				ENDIF
+                IF(intr /= 0) THEN
+                    dl = 2.0D0 * dim(1.0D0, r)
+                    d = dot_product(y,dl)
+                    d = 0.25*d/nobs
+                    IF(d /= 0.0D0) THEN
+                        b(0)=b(0)+d
+                        r=r+y*d
+                        dif=max(dif,d**2)
+                    ENDIF
+                ENDIF
                 IF (ni > pmax) EXIT
                 IF (dif < eps) EXIT
                 IF(npass > maxit) THEN
@@ -1225,15 +1175,11 @@ SUBROUTINE sqsvm_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam
                         g=idx(j)
                         start=ix(g)
                         end=iy(g)
-                        ALLOCATE(u(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        ALLOCATE(dd(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        ALLOCATE(oldb(bs(g)),STAT=ierr)
-                        jerr=jerr+ierr
-                        IF(jerr/=0) RETURN
+                        ALLOCATE(u(bs(g)))
+                        ALLOCATE(dd(bs(g)))
+                        ALLOCATE(oldb(bs(g)))
                         oldb=b(start:end)
-                        dl = 2.0 * dim(1.0, r)
+                        dl = 2.0D0 * dim(1.0D0, r)
                         u=matmul(y*dl,x(:,start:end))/nobs
                         u=gam(g)*b(start:end)+u
                         unorm=sqrt(dot_product(u,u))
@@ -1250,16 +1196,16 @@ SUBROUTINE sqsvm_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam
                         ENDIF
                         DEALLOCATE(u,dd,oldb)
                     ENDDO
-					IF(intr /= 0) THEN
-	                    dl = 2.0 * dim(1.0, r)
-	                    d = dot_product(y,dl)
-	                    d = 0.25*d/nobs
-	                    IF(d/=0.0D0) THEN
-	                        b(0)=b(0)+d
-	                        r=r+y*d
-	                        dif=max(dif,d**2)
-	                    ENDIF
-					ENDIF
+                    IF(intr /= 0) THEN
+                        dl = 2.0D0 * dim(1.0D0, r)
+                        d = dot_product(y,dl)
+                        d = 0.25*d/nobs
+                        IF(d/=0.0D0) THEN
+                            b(0)=b(0)+d
+                            r=r+y*d
+                            dif=max(dif,d**2)
+                        ENDIF
+                    ENDIF
                     IF(dif<eps) EXIT
                     IF(npass > maxit) THEN
                      jerr=-l
@@ -1277,8 +1223,7 @@ SUBROUTINE sqsvm_f (bn,bs,ix,iy,gam,nobs,nvars,x,y,pf,dfmax,pmax,nlam,flmin,ulam
             vl = matmul(dl*y, x) / nobs
             DO g = 1, bn
                 IF(jxx(g) == 1) CYCLE
-                ALLOCATE(u(bs(g)),STAT=ierr)
-                IF(ierr/=0) RETURN
+                ALLOCATE(u(bs(g)))
                 u = vl(ix(g):iy(g))
                 ga(g) = sqrt(dot_product(u,u))
                 IF(ga(g) > al*pf(g))THEN
